# noinspection PyClassHasNoInit
class ClientOption:
    DEBUG = 'debug'
    PID_FILE = 'pid_file'
    LOG_FILE = 'log_file'
    LOG_LEVEL = 'log_level'
    DEVICE_ID = 'device_id'
    DEBUG_PORT = 'debug_port'
    PIN_LOGOUT = 'pin_logout'
    CONFIG_FILE = 'config_file'
    PIN_LED_RED = 'pin_led_red'
    PIN_LED_BLUE = 'pin_led_blue'
    RESTART_DELAY = 'restart_delay'
    PIN_LED_GREEN = 'pin_led_green'
    SERVER_ADDRESS = 'server_address'
    PIN_POWER_RELAY = 'pin_power_relay'
    MINIMUM_UP_TIME = 'minimum_up_time'
    SERIAL_PORT_NAME = 'serial_port_name'
    SERIAL_PORT_SPEED = 'serial_port_speed'
    PIN_CURRENT_SENSE = 'pin_current_sense'
    LOGOUT_COAST_TIME = 'logout_coast_time'
    MAX_RESTART_ATTEMPTS = 'max_restart_attempts'
    LOG_ADDRESS_FOR_PAPER_TRAIL = 'log_address_for_paper_trail'
